-- // Create Contact Directory Schema

-- Contact Directory, if public is false, look to CONTACTDIRECTORYPERMISSIONS table for permissions
CREATE SEQUENCE IF NOT EXISTS CONTACTDIRECTORYSEQ;
CREATE TABLE IF NOT EXISTS CONTACTDIRECTORY
(
   ID    		BIGINT DEFAULT CONTACTDIRECTORYSEQ.NEXTVAL PRIMARY KEY,
   NAME  		NVARCHAR(255),
   OWNER_UUID   VARCHAR(36),
   PUBLIC       BOOLEAN
);

-- Contact Directory Permissions, contains users that have permissions set for a private directory
CREATE SEQUENCE IF NOT EXISTS CONTACTDIRECTORYPERMISSIONSSEQ;
CREATE TABLE IF NOT EXISTS CONTACTDIRECTORYPERMISSIONS
(
	ID 				BIGINT DEFAULT CONTACTDIRECTORYPERMISSIONSSEQ.NEXTVAL PRIMARY KEY,
	DIRECTORY_ID 	BIGINT,
	USER_ID			VARCHAR(36),
	VIEWER			BOOLEAN,
	EDITOR			BOOLEAN
);

-- Contacts
CREATE SEQUENCE IF NOT EXISTS CONTACTSEQ;
CREATE TABLE IF NOT EXISTS CONTACT
(
   ID            BIGINT DEFAULT CONTACTSEQ.NEXTVAL PRIMARY KEY,
   DIRECTORY_ID  BIGINT,
   FOREIGN KEY(DIRECTORY_ID) REFERENCES CONTACTDIRECTORY(ID) ON DELETE CASCADE,
   NAME          NVARCHAR(255),
   COMPANY       NVARCHAR(255),
   JOB_TITLE     NVARCHAR(255)
);

-- Normalized contact subtables
CREATE SEQUENCE IF NOT EXISTS ADDRESSSEQ;
CREATE TABLE IF NOT EXISTS ADDRESS
(
   ID           BIGINT DEFAULT ADDRESSSEQ.NEXTVAL PRIMARY KEY,
   CONTACT_ID   BIGINT,
   FOREIGN KEY(CONTACT_ID) REFERENCES CONTACT(ID) ON DELETE CASCADE,
   LABEL        NVARCHAR(255),
   ADDRESS1     NVARCHAR(1023),
   ADDRESS2     NVARCHAR(1023),
   CITY         NVARCHAR(100),
   STATE        NVARCHAR(255),
   COUNTRY      NVARCHAR(100),
   POSTAL_CODE  NVARCHAR(100)
);

CREATE SEQUENCE IF NOT EXISTS PHONENUMBERSEQ;
CREATE TABLE IF NOT EXISTS PHONENUMBER
(
   ID          BIGINT DEFAULT PHONENUMBERSEQ.NEXTVAL PRIMARY KEY,
   CONTACT_ID  BIGINT,
   FOREIGN KEY(CONTACT_ID) REFERENCES CONTACT(ID) ON DELETE CASCADE,
   LABEL       NVARCHAR(255),
   NUMBER      NVARCHAR(255),
   EXTENSION   NVARCHAR(50)
);

CREATE SEQUENCE IF NOT EXISTS EMAILADDRESSSEQ;
CREATE TABLE IF NOT EXISTS EMAILADDRESS
(
   ID          BIGINT DEFAULT EMAILADDRESSSEQ.NEXTVAL PRIMARY KEY,
   CONTACT_ID  BIGINT,
   FOREIGN KEY(CONTACT_ID) REFERENCES CONTACT(ID) ON DELETE CASCADE,
   LABEL       NVARCHAR(255),
   EADDRESS    NVARCHAR(255)
);

CREATE SEQUENCE IF NOT EXISTS IMADDRESSSEQ;
CREATE TABLE IF NOT EXISTS IMADDRESS
(
	ID          BIGINT DEFAULT IMADDRESSSEQ.NEXTVAL PRIMARY KEY,
   	CONTACT_ID  BIGINT,
    FOREIGN KEY(CONTACT_ID) REFERENCES CONTACT(ID) ON DELETE CASCADE,
   	LABEL		NVARCHAR(255),
   	IMADDRESS	NVARCHAR(255)
);

CREATE SEQUENCE IF NOT EXISTS WEBSITESEQ;
CREATE TABLE IF NOT EXISTS WEBSITE
(
   ID          BIGINT DEFAULT WEBSITESEQ.NEXTVAL PRIMARY KEY,
   CONTACT_ID  BIGINT,
   FOREIGN KEY(CONTACT_ID) REFERENCES CONTACT(ID) ON DELETE CASCADE,
   LABEL       NVARCHAR(255),
   URL         NVARCHAR(1023)
);

CREATE SEQUENCE IF NOT EXISTS NOTESSEQ;
CREATE TABLE IF NOT EXISTS NOTES
(
   ID          BIGINT DEFAULT NOTESSEQ.NEXTVAL PRIMARY KEY,
   CONTACT_ID  BIGINT,
   FOREIGN KEY(CONTACT_ID) REFERENCES CONTACT(ID) ON DELETE CASCADE,
   LABEL       NVARCHAR(255),
   TEXT        CLOB
);

CREATE SEQUENCE IF NOT EXISTS CUSTOMFIELDSSEQ;
CREATE TABLE IF NOT EXISTS CUSTOMFIELDS
(
   ID          BIGINT DEFAULT CUSTOMFIELDSSEQ.NEXTVAL PRIMARY KEY,
   CONTACT_ID  BIGINT,
   FOREIGN KEY(CONTACT_ID) REFERENCES CONTACT(ID) ON DELETE CASCADE,
   KEY         NVARCHAR(255),
   VALUE       NVARCHAR(255)
);

-- //@UNDO

DROP TABLE CONTACTDIRECTORY;
DROP TABLE CONTACTDIRECTORYPERMISSIONS;
DROP TABLE CONTACT;
DROP TABLE ADDRESS;
DROP TABLE PHONENUMBER;
DROP TABLE EMAILADDRESS;
DROP TABLE IMADDRESS;
DROP TABLE WEBSITE;
DROP TABLE NOTES;
DROP TABLE CUSTOMFIELDS;

DROP SEQUENCE CONTACTDIRECTORYSEQ;
DROP SEQUENCE CONTACTDIRECTORYPERMISSIONSSEQ;
DROP SEQUENCE CONTACTSEQ;
DROP SEQUENCE ADDRESSSEQ;
DROP SEQUENCE PHONENUMBERSEQ;
DROP SEQUENCE EMAILADDRESSSEQ;
DROP SEQUENCE IMADDRESSSEQ;
DROP SEQUENCE WEBSITESEQ;
DROP SEQUENCE NOTESSEQ;
DROP SEQUENCE CUSTOMFIELDSSEQ;
